package com.xzzn.ems.service.impl;

import com.xzzn.common.utils.StringUtils;
import com.xzzn.ems.domain.EmsBatteryData;
import com.xzzn.ems.domain.EmsCoolingData;
import com.xzzn.ems.domain.EmsDevicesSetting;
import com.xzzn.ems.domain.vo.*;
import com.xzzn.ems.mapper.*;
import com.xzzn.ems.service.ISingleSiteService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 单点监控 服务层实现
 */
@Service
public class SingleSiteServiceImpl implements ISingleSiteService {

    private static final String CLUSTER_DATA_TEP = "温度";

    private static final String CLUSTER_DATA_VOLTAGE = "电压";

    private static final String CLUSTER_DATA_SOC = "SOC";

    @Autowired
    private EmsPcsDataMapper emsPcsDataMapper;
    @Autowired
    private EmsAlarmRecordsMapper emsAlarmRecordsMapper;
    @Autowired
    private EmsBatteryDataMapper emsBatteryDataMapper;
    @Autowired
    private EmsBatteryStackMapper emsBatteryStackMapper;
    @Autowired
    private EmsBatteryClusterMapper emsBatteryClusterMapper;
    @Autowired
    private EmsPcsBranchDataMapper emsPcsBranchDataMapper;
    @Autowired
    private EmsCoolingDataMapper emsCoolingDataMapper;
    @Autowired
    private EmsAmmeterDataMapper emsAmmeterDataMapper;
    @Autowired
    private EmsDevicesSettingMapper emsDevicesSettingMapper;

    @Override
    public SiteMonitorHomeVo getSiteMonitorDataVo(String siteId) {
        SiteMonitorHomeVo siteMonitorHomeVo = new SiteMonitorHomeVo();

        if (!StringUtils.isEmpty(siteId)) {
            // 获取单站点的总充+总放+电网实时功率
            Map<String,BigDecimal> pcsMap  = emsPcsDataMapper.getPcsTotalChargeData(siteId);
            if (pcsMap != null) {
                siteMonitorHomeVo.setTotalChargedCap(pcsMap.get("totalChargedCap"));
                siteMonitorHomeVo.setTotalDischargedCap(pcsMap.get("totalDischargedCap"));
                siteMonitorHomeVo.setGridNrtPower(pcsMap.get("gridNrtPower"));
            }
            // 实时告警数据 名称+状态+告警内容
            List<SiteMonitorHomeAlarmVo> siteMonitorHomeAlarmVo = emsAlarmRecordsMapper.getAlarmRecordsBySiteId(siteId);
            siteMonitorHomeVo.setSiteMonitorHomeAlarmVo(siteMonitorHomeAlarmVo);
            // 能量数据
            List<SiteMonitorDataVo> siteMonitorDataVoList = emsPcsDataMapper.getPcsDataBySiteId(siteId,6);
            if (!CollectionUtils.isEmpty(siteMonitorDataVoList)) {
                for (SiteMonitorDataVo sitePcsData : siteMonitorDataVoList) {
                    // 获取当天的充电量+放电量
                    String pcsDate = sitePcsData.getAmmeterDate();
                    boolean isToday= checkIsToday(pcsDate);
                    if(isToday){
                        siteMonitorHomeVo.setDayChargedCap(sitePcsData.getChargedCap());
                        siteMonitorHomeVo.setDayDisChargedCap(sitePcsData.getDisChargedCap());
                    }
                }
                // 储能可用电量
                BigDecimal energyStorageAvailElec = siteMonitorHomeVo.getTotalDischargedCap().subtract(siteMonitorHomeVo.getTotalChargedCap());
                siteMonitorHomeVo.setEnergyStorageAvailElec(energyStorageAvailElec);
            }
            siteMonitorHomeVo.setSiteMonitorDataVo(siteMonitorDataVoList);
        }

        return siteMonitorHomeVo;
    }

    private boolean checkIsToday(String pcsDate) {
        boolean flag = false;
        Calendar calendar = Calendar.getInstance();
        int todayMonth = calendar.get(Calendar.MONTH) + 1;
        int todayDay = calendar.get(Calendar.DAY_OF_MONTH);
        if (StringUtils.isNotEmpty(pcsDate)){
            String[] pcsDateArray = pcsDate.split("/");
            if (todayMonth == Integer.parseInt(pcsDateArray[0]) &&
                    todayDay == Integer.parseInt(pcsDateArray[1])) {
                flag = true;
            }
        }
        return flag;
    }

    // 获取单站监控实时运行头部数据
    @Override
    public SiteMonitorRunningHeadInfoVo getSiteRunningHeadInfo(String siteId) {
        SiteMonitorRunningHeadInfoVo siteMonitorRunningHeadInfoVo = new SiteMonitorRunningHeadInfoVo();

        if (!StringUtils.isEmpty(siteId)) {
            // 实时有功功率/实时无功功率/今日充电量/今日放电量
            siteMonitorRunningHeadInfoVo = emsPcsDataMapper.getSiteRunningHeadInfo(siteId);
            // 电池簇SOC
            // 电池堆SOH
            EmsBatteryData emsBatteryData = emsBatteryDataMapper.getBatteryDataBySiteId(siteId);
            if (emsBatteryData != null) {
                siteMonitorRunningHeadInfoVo.setSoc(emsBatteryData.getSoc());
                siteMonitorRunningHeadInfoVo.setSoh(emsBatteryData.getSoh());
            }
        }

        return siteMonitorRunningHeadInfoVo;
    }

    //  获取单站监控实时运行曲线图数据
    @Override
    public SiteMonitorRuningInfoVo getRunningGraph(String siteId) {
        SiteMonitorRuningInfoVo siteMonitorRuningInfoVo = new SiteMonitorRuningInfoVo();
        if (!StringUtils.isEmpty(siteId)) {
            //储能功率list
            //pcs平均温度list
            //电池平均soclist
            //电池平均温度list
        }
        return siteMonitorRuningInfoVo;
    }

    // 根据site_id获取pcs详细数据+支路数据
    @Override
    public List<PcsDetailInfoVo> getPcsDetailInfo(String siteId) {
        List<PcsDetailInfoVo> pcsDetailInfoVoList = new ArrayList<>();

        if (!StringUtils.isEmpty(siteId)) {
            // 获取pcs最新数据
            pcsDetailInfoVoList = emsPcsDataMapper.getPcsDetailInfoBySiteId(siteId);
            if (!CollectionUtils.isEmpty(pcsDetailInfoVoList)) {
                for (PcsDetailInfoVo pcsDetailInfoVo : pcsDetailInfoVoList) {
                    String deviceId = pcsDetailInfoVo.getDeviceId();
                    // 获取支路最新数据
                    if (!StringUtils.isEmpty(deviceId)) {
                        List<PcsBranchInfo> pcsBranchInfoList = emsPcsBranchDataMapper.getPcsBranchInfoList(siteId, deviceId);
                        pcsDetailInfoVo.setPcsBranchInfoList(pcsBranchInfoList);
                    }
                }
            }
        }
        return pcsDetailInfoVoList;
    }

    // 获取BMS总览数据
    @Override
    public List<BMSOverViewVo> getBMSOverView(String siteId) {
        List<BMSOverViewVo> bmsOverViewVoList = new ArrayList<>();

        if (!StringUtils.isEmpty(siteId)) {
            // 获取电池堆list
            bmsOverViewVoList  = emsBatteryStackMapper.selectEmsBatteryStackBySiteId(siteId);
            if (!CollectionUtils.isEmpty(bmsOverViewVoList)) {
                for (BMSOverViewVo bmsOverViewVo : bmsOverViewVoList) {
                    // 获取单体电池数据
                    String stackDeviceId = bmsOverViewVo.getDeviceId();
                    if (!StringUtils.isEmpty(stackDeviceId)) {
                        List<BMSBatteryDataList> batteryDataList = emsBatteryClusterMapper.getBmsBatteryData(siteId,stackDeviceId);
                        bmsOverViewVo.setBatteryDataList(batteryDataList);
                    }
                }
            }
        }
        return bmsOverViewVoList;
    }

    // 获取BMS电池簇数据
    @Override
    public List<BMSBatteryClusterVo> getBMSBatteryCluster(String siteId) {
        List<BMSBatteryClusterVo> bmsBatteryClusterVoList = new ArrayList<>();

        if (!StringUtils.isEmpty(siteId)) {
            bmsBatteryClusterVoList = emsBatteryClusterMapper.getBMSBatteryCluster(siteId);
            if (!CollectionUtils.isEmpty(bmsBatteryClusterVoList)) {
                for (BMSBatteryClusterVo bmsBatteryClusterVo : bmsBatteryClusterVoList) {

                    String clusterDeviceId = bmsBatteryClusterVo.getDeviceId();
                    List<BMSBatteryClusterDataList> clusterDataList = new ArrayList<>();
                    if (!StringUtils.isEmpty(clusterDeviceId)) {
                        // 获取单体电池数据-平均/最大/最小
                        //BatteryClusterDataDetailVo  batteryClusterDataDetailVo = emsBatteryDataMapper.getBatteryDataByClusterId(siteId,clusterDeviceId);
                        // 处理数据
                        BMSBatteryClusterDataList voltageData = new BMSBatteryClusterDataList();
                        BMSBatteryClusterDataList tempData = new BMSBatteryClusterDataList();
                        BMSBatteryClusterDataList socData = new BMSBatteryClusterDataList();
                        // 设值
                        voltageData.setDataName(CLUSTER_DATA_VOLTAGE);
                        voltageData.setAvgData(bmsBatteryClusterVo.getAvgVoltage());
                        voltageData.setMaxData(bmsBatteryClusterVo.getMaxVoltage());
                        voltageData.setMaxDataID(bmsBatteryClusterVo.getMaxVoltageID());
                        voltageData.setMinData(bmsBatteryClusterVo.getMinVoltage());
                        voltageData.setMinDataID(bmsBatteryClusterVo.getMinVoltageID());
                        tempData.setDataName(CLUSTER_DATA_TEP);
                        tempData.setAvgData(bmsBatteryClusterVo.getAvgTemp());
                        tempData.setMaxData(bmsBatteryClusterVo.getMaxTemp());
                        tempData.setMaxDataID(bmsBatteryClusterVo.getMaxTempID());
                        tempData.setMinData(bmsBatteryClusterVo.getMinTemp());
                        tempData.setMinDataID(bmsBatteryClusterVo.getMinTempID());
                        socData.setDataName(CLUSTER_DATA_SOC);
                        socData.setAvgData(bmsBatteryClusterVo.getAvgSoc());
                        socData.setMaxData(bmsBatteryClusterVo.getMaxSoc());
                        socData.setMaxDataID(bmsBatteryClusterVo.getMaxSocID());
                        socData.setMinData(bmsBatteryClusterVo.getMinSoc());
                        socData.setMinDataID(bmsBatteryClusterVo.getMinSocID());

                        clusterDataList.add(voltageData);
                        clusterDataList.add(tempData);
                        clusterDataList.add(socData);
                        bmsBatteryClusterVo.setBatteryDataList(clusterDataList);
                    }
                }
            }
        }
        return bmsBatteryClusterVoList;
    }

    // 获取液冷设备参数
    @Override
    public  List<EmsCoolingData> getCoolingDataList(String siteId) {
        List<EmsCoolingData> emsCoolingDataList = new ArrayList<>();
        if (!StringUtils.isEmpty(siteId)) {
            emsCoolingDataList = emsCoolingDataMapper.getCoolingDataList(siteId);
        }
        return emsCoolingDataList;
    }


    /**
     * 根据电池堆获取单体柱状数据
     * @param clusterDeviceId
     * @return
     */
    @Override
    public  List<BatteryDataStatsListVo> getClusterDataInfoList(String clusterDeviceId) {
        List<BatteryDataStatsListVo> batteryDataStatsListVo = new ArrayList<>();
        batteryDataStatsListVo = emsBatteryDataMapper.getAllBatteryDataByClusterId(clusterDeviceId);
        return batteryDataStatsListVo;
    }

    /**
     * 电表数据信息
     * @param siteId
     * @return
     */
    @Override
    public List<AmmeterDataVo> getAmmeterDataList(String siteId) {
        List<AmmeterDataVo> ammeterDataVos = new ArrayList<>();

        if (!StringUtils.isEmpty(siteId)) {
            // 先获取所有电表设备
            List<EmsDevicesSetting> devicesList = emsDevicesSettingMapper.getAllBatteryDeviceBySiteId(siteId);
            if (!CollectionUtils.isEmpty(devicesList)) {
                for (EmsDevicesSetting devicesSetting : devicesList) {
                    AmmeterDataVo ammeterDataVo = new AmmeterDataVo();
                    ammeterDataVo.setDeviceName(devicesSetting.getDeviceName());
                    ammeterDataVo.setEmsCommunicationStatus(devicesSetting.getCommunicationStatus());
                    String deviceId = devicesSetting.getDeviceId();
                    // 获取类别数据
                    List<AmmeterDataDetailInfo> ammeterDataDetailInfos = emsAmmeterDataMapper.getAmmeterDetailInfo(siteId,deviceId);
                    ammeterDataVo.setAmmeterDataDetailInfos(ammeterDataDetailInfos);
                    // 数据更新时间
                    ammeterDataVo.setDataUpdateTime(ammeterDataDetailInfos.get(0).getUpdateTime());

                    ammeterDataVos.add(ammeterDataVo);
                }
            }
        }
        return ammeterDataVos;
    }

}
